#include	"DSPamPi.h"
#include	"DisplayOled.h"

#define COUNT_CHANNELS                            5
#define MUTE_PORT                                 9
#define MUTE_INIT_VALUE                           0
#define STANDBY_PORT                              8
#define STANDBY_INIT_VALUE                        0
#define DISPLAY_OFF_DELAY_TIME                    2000     // Millisekunden
#define VOLUME_SAVE_TIME                          3000
#define COUNT_SAVE_COMMANDS						  25

#define	DISPLAY_ON_OFF	if (isDisplayOn == false) {	pDisplay->displayOn(); 	lastDisplayOffActivity = millis(); }


DSPamPi::DSPamPi()
{
	pDisplay = new DisplayOled();
	pInitValues = new InitValues();
	pX9C103 = new X9C103();
	pControls = new Controls();
}


DSPamPi::~DSPamPi()
{
}


void DSPamPi::init(String softwareVersionPar, String compatibleParameterVersions[])
{
	softwareVersion = softwareVersionPar;
	Serial.println(softwareVersion);
	pDisplay->init(softwareVersion);
	if (checkParameterVersion(*pInitValues->getParameterVersion(), compatibleParameterVersions) == false)
	{
		char achBuffer[30];

		sprintf(achBuffer, "%s %s", softwareVersion.c_str(), pInitValues->getParameterVersion()->c_str());
		pDisplay->error((char *)"Setup notwendig", achBuffer);
		for (;;)
			delay(10);
	}
	Serial.print("Max Volume:  ");
	Serial.println(pInitValues->getMaxVolume());
	mutePort.init(MUTE_PORT, MUTE_INIT_VALUE);
	standbyPort.init(STANDBY_PORT, STANDBY_INIT_VALUE);
	isMute = false;
	isStandby = false;
	isDisplayOn = true;
	pX9C103->initVorStandbyeHigh();
	standbyPort.On();
	delay(4000);
	pX9C103->initNachStandbyeHigh(0);
	pX9C103->clear();
//	mutePort.On();
	pControls->init(pInitValues->pRc5Commands);
	volumeFromDefault();
	channelRelay.init();
	channel = pInitValues->channel;
	channelAbsolutInit(channel);
	Serial.print("Channel: ");
	Serial.println(channel);
	delay(500);
	pX9C103->set(volume[channel]);
	pDisplay->displayVolume(volume[channel], isMute);
	pDisplay->displayChannel(*pInitValues->getChannelName(channel));
	mutePort.On();
//	lautstaerkeAbsolut(volume[channel], true, channel);
//	channelAbsolutInit(channel);
	lastVolumeSaveTime = 0;
}


void DSPamPi::work(void)
{
	Action          *pAction = pControls->getAction();
	int             newChannel = 0;
	int             newVolume = 0;

	switch (pAction->getActionType())
	{
		case ACTION_POWER:																						// Standby
			Serial.println("DSPamPI::work ACTION_POWER action erhalten");
			if (isStandby == true)
			{
				mutePort.init(MUTE_PORT, MUTE_INIT_VALUE);
				standbyPort.init(STANDBY_PORT, STANDBY_INIT_VALUE);
				isMute = false;
				isStandby = false;
				isDisplayOn = true;


				delete pInitValues;

				pInitValues = new InitValues();


				pX9C103->initVorStandbyeHigh();
				standbyPort.On();
				pDisplay->displayOn();
				pDisplay->displayStart(softwareVersion);
				delay(4000);
				pX9C103->initNachStandbyeHigh(0);
				pX9C103->clear();
//				mutePort.On();
				pControls->init(pInitValues->pRc5Commands);
				volumeFromDefault();
				channelRelay.init();
				newChannel = pInitValues->channel;
				channelAbsolutInit(newChannel);
				Serial.print("Channel: ");
				Serial.println(newChannel);
				delay(500);
				pX9C103->set(volume[newChannel]);
				pDisplay->displayVolume(volume[newChannel], isMute);
				pDisplay->displayChannel(*pInitValues->getChannelName(newChannel));
				mutePort.On();
//				lautstaerkeAbsolut(volume[newChannel], true, channel);
//				channelAbsolutInit(newChannel);
				lastVolumeSaveTime = 0;
			}
			else
			{
				pControls->getButton()->setDummyInterrupt();
				pControls->getRotaryEncoder()->setDummyInterrupt();
				isStandby = true;
				pDisplay->displayOff();
				if (isMute == true)
					changeMute();
				standbyPort.Off();
				mutePort.Off();
			}
			break;

		case VOLUME_MINUS:
			Serial.println("DSPamPI::work VOLUME_MINUS action erhalten");
		case ROTATE_LEFT:
			DISPLAY_ON_OFF
			newVolume = lautstaerkeRelativ(-1, channel);
			pDisplay->displayVolume(newVolume, isMute);
			break;

		case VOLUME_PLUS:
			Serial.println("DSPamPI::work VOLUME_PLUS action erhalten");
		case ROTATE_RIGHT:
			DISPLAY_ON_OFF
			newVolume = lautstaerkeRelativ(1, channel);
			pDisplay->displayVolume(newVolume, isMute);
			break;

		case VOLUME_CLEAR:
			Serial.println("DSPamPI::work VOLUME_CLEAR action erhalten");
			DISPLAY_ON_OFF
			pDisplay->displayVolume(0, isMute);
			pX9C103->clear();
			break;


		case PUSH_BUTTON_LONG:
			// ?????
			Serial.println("DSPamPI::work PUSH_BUTTON_LONG action erhalten");
			break;

		case CHANNEL_MINUS:
			Serial.println("DSPamPI::work CHANNEL_MINUS action erhalten");
			DISPLAY_ON_OFF
			pX9C103->set(0);
			channelRelay.deselectAllChannels();
			newChannel = channelRelativ(-1);
			pDisplay->displayChannel(*pInitValues->getChannelName(newChannel));
			channelRelay.switchRelays(newChannel);
			pX9C103->set(volume[newChannel]);
			pInitValues->saveNVRamChannel(newChannel);
			pDisplay->displayVolume(volume[newChannel], isMute);
			break;

		case CHANNEL_PLUS:
			DISPLAY_ON_OFF
			Serial.println("DSPamPI::work CHANNEL_PLUS action erhalten");
			pX9C103->set(0);
			channelRelay.deselectAllChannels();
			newChannel = channelRelativ(1);
			pDisplay->displayChannel(*pInitValues->getChannelName(newChannel));
			channelRelay.switchRelays(newChannel);
			pX9C103->set(volume[newChannel]);
			pInitValues->saveNVRamChannel(newChannel);
			pDisplay->displayVolume(volume[newChannel], isMute);
			break;

		case CHANNEL_ABSOLUTE:
			Serial.println("DSPamPI::work CHANNEL_ABSOLUTE action erhalten");
			DISPLAY_ON_OFF
			if (channel != pAction->getChannelAbsolute())
			{
				pX9C103->set(0);
				newChannel = channelAbsolut(pAction->getChannelAbsolute());
				pX9C103->set(volume[newChannel]);
				pDisplay->displayChannel(*pInitValues->getChannelName(newChannel));
				pInitValues->saveNVRamChannel(newChannel);
				pDisplay->displayVolume(volume[newChannel], isMute);
			}
			break;

		case MUTE:
			Serial.println("DSPamPI::work MUTE action erhalten");
		case PUSH_BUTTON_SHORT:
			DISPLAY_ON_OFF
			changeMute();
			break;

		case VOLUME_SAVE:
			Serial.println("DSPamPI::work VOLUME_SAVE action erhalten");
			DISPLAY_ON_OFF
			if (isVolumeSave((unsigned long)(millis())) == true)
			{
				pInitValues->saveNVRamVolumeChannel(channel, volume[channel]);
				displayVolumeSave(volume[channel]);
			}
			break;

		case VOLUME_RESTORE:
			Serial.println("DSPamPI::work VOLUME_RESTORE action erhalten");
			DISPLAY_ON_OFF
			newVolume = pInitValues->getChannelDefaultValue(channel);
			pX9C103->set(newVolume, volume[channel]);
			volume[channel] = newVolume;
			pDisplay->displayVolume(newVolume, isMute);
			break;


		case ACTION_DISPLAY:
			Serial.println("DSPamPI::work ACTION_DISPLAY action erhalten");
			if (isDisplayOn == true)
			{
				isDisplayOn = false;
				pDisplay->displayOff();
			}
			else
			{
				isDisplayOn = true;
				pDisplay->displayOn();
			}
			break;
	}
	if (isDisplayOn == false)
	{

		long    millisecs = millis();

		if (!(millisecs < (lastDisplayOffActivity + DISPLAY_OFF_DELAY_TIME)))
		{
			pDisplay->displayOff();
		}
	}
	delete pAction;
}


int DSPamPi::channelAbsolutInit(int channel)
{
	if ((channel >= 0) && (channel <= COUNT_CHANNELS - 1))
	{
		channelRelay.switchRelays(channel);
	}
	return channel;
}


int DSPamPi::channelAbsolut(int newValue)
{
	int   actualChannel = channel;
	int   newChannel = channel;

	if ((newValue >= 0) && (newValue <= COUNT_CHANNELS - 1))
	{
		if (newValue != actualChannel)
		{
			channel = newValue;
			channelRelay.deselectAllChannels();
//			volumeRelay.switchRelays(volume[channel]);
			channelRelay.switchRelays(channel);
			pInitValues->saveNVRamChannel(channel);
		}
	}
	return channel;
}

int DSPamPi::channelRelativ(int delta)
{
	int   actualChannel = channel;

	if (delta > 0)
	{
		if (channel + delta > COUNT_CHANNELS - 1)
		{
			channel = 0;
		}
		else
		{
			channel = channel + delta;
		}
	}
	else
	{
		if (channel + delta < 0)
		{
			channel = 4;
		}
		else
		{
			channel = channel + delta;
		}
	}
	return channel;
}

void    DSPamPi::changeMute()
{
	if (isMute)
	{
		isMute = false;
		pDisplay->displayVolume(volume[channel], isMute);
		pDisplay->displayChannel(*pInitValues->getChannelName(channel));
		mutePort.On();
	}
	else
	{
		isMute = true;
		pDisplay->displayVolume(volume[channel], isMute);
		mutePort.Off();
	}
}


bool DSPamPi::checkParameterVersion(String parameterVersion, String compatibleParameterVersions[])
{
	//  Serial.println(parameterVersion.c_str());
	for (int i = 0; ; i++)
	{
		//    Serial.println(compatibleParameterVersions[i].c_str());
		if (compatibleParameterVersions[i] == "")
			return false;
		if (parameterVersion == compatibleParameterVersions[i])
			return true;
	}
	return false;
}

void    DSPamPi::volumeFromDefault()
{
	for (int i = 0; i < 5; i++)
		volume[i] = pInitValues->getChannelDefaultValue(i);
}

/*
int DSPamPi::lautstaerkeAbsolut(int newValue, bool saveValue, int channel)
{
	if ((0 <= newValue) && (newValue <= pInitValues->getMaxVolume()))
	{
		if (saveValue)
		{
			volume[channel] = newValue;
		}
		pX9C103->set(newValue);
		return newValue;
	}
	else
		return volume[channel];
}
*/

int DSPamPi::lautstaerkeRelativ(int delta, int channel)
{
	if (delta > 0)
	{
		if (volume[channel] + delta > pInitValues->getMaxVolume())		// Wird nur mit delta 1 aufgerufen
		{
//			volume[channel] = pInitValues->getMaxVolume();
			return;														
		}
		else
		{
			volume[channel] = volume[channel] + delta;
		}
		pX9C103->increment(delta);
	}
	else
	{
		if (volume[channel] + delta < 0)								// Wird nur mit delta -1 aufgerufen
		{
//			volume[channel] = 0;
			return;
		}
		else
		{
			volume[channel] = volume[channel] + delta;
		}
		pX9C103->decrement(-delta);
	}
//	delay(VOLUME_DELAY);
	return volume[channel];
}


void DSPamPi::displayVolumeSave(int volume)
{
	for (int i = 0; i < 3; i++)
	{
		pDisplay->clearVolume();
		delay(500);
		pDisplay->displayVolume(volume, false);
		delay(500);
	}
}


void DSPamPi::resetVolumeSave()
{
	lastVolumeSaveTime = 0;
	countSaveCommands = 0;
}


bool DSPamPi::isVolumeSave(unsigned long lastVolumeSaveTimePar)
{
	if (countSaveCommands == 0)
	{
		lastVolumeSaveTime = millis();
	}
	countSaveCommands++;
	if (((lastVolumeSaveTimePar - lastVolumeSaveTime) > VOLUME_SAVE_TIME) && (countSaveCommands >= COUNT_SAVE_COMMANDS))
	{
		resetVolumeSave();
		return true;
	}
	else
	{
		if ((lastVolumeSaveTimePar - lastVolumeSaveTime) > VOLUME_SAVE_TIME)
		{
			resetVolumeSave();
		}
		return false;
	}
}
