#include <Arduino.h>
#include "DisplayOled.h"

#define   LCD_RS_PIN      40
#define   LCD_RW_PIN      42
#define   LCD_ENABLE_PIN  44
#define   LCD_DATA_4      46
#define   LCD_DATA_5      48
#define   LCD_DATA_6      50
#define   LCD_DATA_7      52


#define MIN_VOLUME        -57
//#define MIN_VOLUME        -51

#define   RC_BLINK_CHAR   '.'

String leerStringOled = "                ";

DisplayOled::DisplayOled()
{
}


DisplayOled::~DisplayOled()
{
}

void  DisplayOled::init(String softwareVersion)
{

	//  Serial.println("DisplayDisplayOled::init Software Version: " + softwareVersion);  
	//  Serial.println("DisplayDisplayOled::init");  

	InitOled();
	displayStart(softwareVersion);
}

void DisplayOled::displayStart(String softwareVersion)
{
	String  meldung = "DSPamPi";
	int     offset = getStartPosition(meldung);

	this->setCursor(0, 0);
	this->print(leerStringOled.c_str());
	this->setCursor(offset, 0);
	this->print(meldung.c_str());

	String  software = "S/W rev. " + softwareVersion;

	offset = getStartPosition(software);
	this->setCursor(0, 1);
	this->print(leerStringOled.c_str());
	this->setCursor(offset, 1);
	this->print(software.c_str());
}


void DisplayOled::error(char *zeile1, char *zeile2)
{
	InitOled();
	this->setCursor(0, 0);
	this->print(leerStringOled.c_str());
	this->setCursor(0, 0);
	this->print(zeile1);

	this->setCursor(0, 1);
	this->print(leerStringOled.c_str());
	this->setCursor(0, 1);
	this->print(zeile2);
}



void DisplayOled::setCursor(int column, int row)
{
	int offset = row * 0x40 + column;
	WriteIns(offset | 0x80);
}

void DisplayOled::displayVolume(int volume, bool isMute)
{
	char  displayBuffer[50];
	int displayVolume = volume; // +MIN_VOLUME;

	volumeSave = volume;
	isMuteSave = isMute;
	// 1.16 Volume springt nicht mehr. Zahlen und dB sind jetzt rechtsbndig
	if (isMute)
		sprintf(displayBuffer, "Mute %3d", displayVolume);
	else
		sprintf(displayBuffer, "Volume %3d", displayVolume);
	this->setCursor(0, 1);
	this->print(leerStringOled.c_str());
	int offset = getStartPosition(displayBuffer);
	this->setCursor(offset, 1);
	this->print(displayBuffer);
}

void DisplayOled::clearVolume()
{
	this->setCursor(0, 1);
	this->print(leerStringOled.c_str());
}


void DisplayOled::displayChannel(String channel)
{
	channelSave = channel;
	String channelMitUmlauten = utf8ascii(channel);

	this->setCursor(0, 0);
	this->print(leerStringOled.c_str());

	int offset = getStartPosition(channelMitUmlauten);
	this->setCursor(offset, 0);
	this->print(channelMitUmlauten.c_str());
}

void DisplayOled::displayString(int line, String value)
{
	String valueMitUmlauten = utf8ascii(value);

	this->setCursor(0, line);
	this->print(leerStringOled.c_str());

	int offset = getStartPosition(valueMitUmlauten);
	this->setCursor(offset, line);
	this->print(valueMitUmlauten.c_str());
}


void DisplayOled::displayOff()
{
	delayMicroseconds(50);
	WriteIns(0x08);     // Display off
}


void DisplayOled::displayOn()
{
	delayMicroseconds(50);
	WriteIns(0x0c);     // Display on
	displayVolume(volumeSave, isMuteSave);
	displayChannel(channelSave);
}


void DisplayOled::InitOled()
{

	pinMode(LCD_RS_PIN, OUTPUT);
	pinMode(LCD_RW_PIN, OUTPUT);
	pinMode(LCD_ENABLE_PIN, OUTPUT);

	pinMode(LCD_DATA_4, OUTPUT);
	pinMode(LCD_DATA_5, OUTPUT);
	pinMode(LCD_DATA_6, OUTPUT);
	pinMode(LCD_DATA_7, OUTPUT);

	digitalWrite(LCD_RS_PIN, 0);
	digitalWrite(LCD_RW_PIN, 0);
	digitalWrite(LCD_ENABLE_PIN, 0);

	delay(40);
	SendNibble(0x03);
	delay(40);
	SendNibble(0x03);
	delayMicroseconds(150);
	SendNibble(0x03);
	delayMicroseconds(150);
	SendNibble(0x02);

	delayMicroseconds(50);
	WriteIns(0x29);     // 4-Bit European Charset
	WriteIns(0x08);     // Display off
	WriteIns(0x06);
	WriteIns(0x17);
	WriteIns(0x01);     // Clear Display
	delay(7);
	WriteIns(0x02);     // Return home
	WriteIns(0x0c);     // Display on
}

void DisplayOled::WriteIns(unsigned char instruction)
{
	pinMode(LCD_DATA_4, OUTPUT);
	pinMode(LCD_DATA_5, OUTPUT);
	pinMode(LCD_DATA_6, OUTPUT);
	pinMode(LCD_DATA_7, OUTPUT);
	SendNibble((instruction & 0xf0) >> 4);
	SendNibble((instruction & 0x0f));
}

void DisplayOled::SendNibble(unsigned char wert)
{
	digitalWrite(LCD_RS_PIN, 0);
	digitalWrite(LCD_RW_PIN, 0);
	digitalWrite(LCD_DATA_7, (wert & 0x08) >> 3);
	digitalWrite(LCD_DATA_6, (wert & 0x04) >> 2);
	digitalWrite(LCD_DATA_5, (wert & 0x02) >> 1);
	digitalWrite(LCD_DATA_4, (wert & 0x01));
	digitalWrite(LCD_ENABLE_PIN, 1);
	delayMicroseconds(10);
	digitalWrite(LCD_ENABLE_PIN, 0);
	delayMicroseconds(10);
}


void DisplayOled::SendNibbleData(unsigned char wert)
{
	digitalWrite(LCD_RS_PIN, 1);
	digitalWrite(LCD_RW_PIN, 0);
	digitalWrite(LCD_DATA_7, (wert & 0x08) >> 3);
	digitalWrite(LCD_DATA_6, (wert & 0x04) >> 2);
	digitalWrite(LCD_DATA_5, (wert & 0x02) >> 1);
	digitalWrite(LCD_DATA_4, (wert & 0x01));
	digitalWrite(LCD_ENABLE_PIN, 1);
	delayMicroseconds(10);
	digitalWrite(LCD_ENABLE_PIN, 0);
	delayMicroseconds(10);
}


unsigned char DisplayOled::ReadNibbleData()
{
	unsigned char wert;

	digitalWrite(LCD_RS_PIN, 0);
	digitalWrite(LCD_RW_PIN, 1);
	digitalWrite(LCD_ENABLE_PIN, 0);
	delayMicroseconds(10);
	wert = digitalRead(LCD_DATA_7) << 3;
	wert |= digitalRead(LCD_DATA_6) << 2;
	wert |= digitalRead(LCD_DATA_5) << 1;
	wert |= digitalRead(LCD_DATA_4);
	digitalWrite(LCD_ENABLE_PIN, 1);
	delayMicroseconds(10);
	return wert;
}


size_t DisplayOled::write(uint8_t zeichen)
{
	//  Serial.println(zeichen);
	SendNibbleData((zeichen & 0xf0) >> 4);
	SendNibbleData(zeichen & 0x0f);
	return 1;
}


int DisplayOled::getStartPosition(String displayString)
{
	int length = strlen(displayString.c_str());

	int pos = (16 - length) / 2;

	return pos;
}

void DisplayOled::print(const char *output)
{
	while (*output)
	{
		write(*output);
		output++;
	}
}

static byte c1;  // Last character buffer

				 // Convert a single Character from UTF8 to Extended ASCII
				 // Return "0" if a byte has to be ignored
byte DisplayOled::utf8ascii(byte ascii) {
	if (ascii<128)   // Standard ASCII-set 0..0x7F handling  
	{
		c1 = 0;
		return(ascii);
	}

	// get previous input
	byte last = c1;   // get last char
	c1 = ascii;         // remember actual character

	switch (last)     // conversion depending on first UTF8-character
	{
	case 0xC2:
		return  (ascii);
		break;

	case 0xC3:
		switch (c1)
		{
		case 0x9f:    // 
			return 0xc3;
			break;

		case 0xb6:    // 
			return 0x8e;
			break;

		case 0xa4:    // 
			return 0x9e;
			break;

		case 0xbc:    // 
			return 0xb2;
			break;

		case 0x96:    // 
			return 0xb1;
			break;

		case 0x84:    // 
			return 0x99;
			break;

		case 0x9c:    // 
			return 0x83;
			break;
		}
		break;
	}

	return  (0);                                     // otherwise: return zero, if character has to be ignored
}
String DisplayOled::utf8ascii(String s)
{
	String r = "";
	char c;
	for (int i = 0; i<s.length(); i++)
	{
		c = utf8ascii(s.charAt(i));
		if (c != 0) r += c;
	}
	return r;
}

void  DisplayOled::rcFlash(bool on)
{
	char achBuffer[2] = { 0, 0 };

	achBuffer[0] = RC_BLINK_CHAR;
	if (rcFlashDisplay != on)
	{
		this->setCursor(15, 1);
		this->print(on == true ? achBuffer : " ");
		rcFlashDisplay = on;
	}
}
