#include "InitValues.h"
#include "EEPROM.h"

#define     OFFSET_VERSION                0
#define     OFFSET_CHANNEL                6
#define     OFFSET_RC5_RC6                8
#define     OFFSET_VOLUME_CHANNEL_0       10
#define     OFFSET_VOLUME_CHANNEL_1       12
#define     OFFSET_VOLUME_CHANNEL_2       14
#define     OFFSET_VOLUME_CHANNEL_3       16
#define     OFFSET_VOLUME_CHANNEL_4       18
#define     OFFSET_LABEL_CHANNEL_0        20
#define     OFFSET_LABEL_CHANNEL_1        40
#define     OFFSET_LABEL_CHANNEL_2        60
#define     OFFSET_LABEL_CHANNEL_3        80
#define     OFFSET_LABEL_CHANNEL_4        100
#define     OFFSET_RC5_COMMANDS           120
#define     OFFSET_RC6_COMMANDS           220
#define		OFFSET_MAX_VOLUME			  320


extern int   anzahlRC5Commands;


InitValues::InitValues(String parameterVersionPar, int channelPar, int rc5Rc6Par,
	String labelChannel1Par, int volumeChannel1Par,
	String labelChannel2Par, int volumeChannel2Par,
	String labelChannel3Par, int volumeChannel3Par,
	String labelChannel4Par, int volumeChannel4Par,
	String labelChannel5Par, int volumeChannel5Par,
	int maxVolumePar,
	RC5_COMMAND_VALUES *rc5CommandsPar)
{
	Serial.println("InitValues::InitValues mit Par");

	EEPROM.write(OFFSET_VERSION, parameterVersionPar[0]);
	EEPROM.write(OFFSET_VERSION + 1, parameterVersionPar[1]);
	EEPROM.write(OFFSET_VERSION + 2, parameterVersionPar[2]);
	EEPROM.write(OFFSET_VERSION + 3, parameterVersionPar[3]);
	EEPROM.write(OFFSET_VERSION + 4, parameterVersionPar[4]);

	EEPROM.write(OFFSET_CHANNEL, channelPar);
	EEPROM.write(OFFSET_RC5_RC6, rc5Rc6Par);

	Serial.println("InitValues::InitValues mit Par 2");
	EEPROM.write(OFFSET_VOLUME_CHANNEL_0, volumeChannel1Par);
	EEPROM.write(OFFSET_VOLUME_CHANNEL_1, volumeChannel2Par);
	EEPROM.write(OFFSET_VOLUME_CHANNEL_2, volumeChannel3Par);
	EEPROM.write(OFFSET_VOLUME_CHANNEL_3, volumeChannel4Par);
	EEPROM.write(OFFSET_VOLUME_CHANNEL_4, volumeChannel5Par);

	//  Serial.println("InitValues::InitValues mit Par 3");
	writeLabel(OFFSET_LABEL_CHANNEL_0, labelChannel1Par);
	writeLabel(OFFSET_LABEL_CHANNEL_1, labelChannel2Par);
	writeLabel(OFFSET_LABEL_CHANNEL_2, labelChannel3Par);
	writeLabel(OFFSET_LABEL_CHANNEL_3, labelChannel4Par);
	writeLabel(OFFSET_LABEL_CHANNEL_4, labelChannel5Par);
	//  Serial.println("InitValues::InitValues mit Par 4");

	writeRc5Commands(OFFSET_RC5_COMMANDS, rc5CommandsPar);
	//  Serial.println("InitValues::InitValues mit Par 5");

	EEPROM.write(OFFSET_MAX_VOLUME, maxVolumePar);
}

InitValues::InitValues()
{
	char    versionBuffer[6];
	//  Serial.println("InitValues::InitValues");
	versionBuffer[0] = EEPROM.read(OFFSET_VERSION);
	versionBuffer[1] = EEPROM.read(OFFSET_VERSION + 1);
	versionBuffer[2] = EEPROM.read(OFFSET_VERSION + 2);
	versionBuffer[3] = EEPROM.read(OFFSET_VERSION + 3);
	versionBuffer[4] = EEPROM.read(OFFSET_VERSION + 4);
	versionBuffer[5] = '\0';
	pParameterVersion = new String(versionBuffer);
	//  Serial.println("InitValues::InitValues2");
	//  Serial.println(pSoftwareVersion->c_str());

	channel = EEPROM.read(OFFSET_CHANNEL);
	rc5Rc6 = EEPROM.read(OFFSET_RC5_RC6);

	//  Serial.println("InitValues::InitValues3");
	volumeChannel1 = EEPROM.read(OFFSET_VOLUME_CHANNEL_0);
	volumeChannel2 = EEPROM.read(OFFSET_VOLUME_CHANNEL_1);
	volumeChannel3 = EEPROM.read(OFFSET_VOLUME_CHANNEL_2);
	volumeChannel4 = EEPROM.read(OFFSET_VOLUME_CHANNEL_3);
	volumeChannel5 = EEPROM.read(OFFSET_VOLUME_CHANNEL_4);
	//  Serial.println("InitValues::InitValues4");

	pLabelChannel1 = readLabel(OFFSET_LABEL_CHANNEL_0);
	pLabelChannel2 = readLabel(OFFSET_LABEL_CHANNEL_1);
	pLabelChannel3 = readLabel(OFFSET_LABEL_CHANNEL_2);
	pLabelChannel4 = readLabel(OFFSET_LABEL_CHANNEL_3);
	pLabelChannel5 = readLabel(OFFSET_LABEL_CHANNEL_4);
	//  Serial.println("InitValues::InitValues5");

	pRc5Commands = readRc5Commands(OFFSET_RC5_COMMANDS);
	//  Serial.println("InitValues::InitValues6");

	maxVolume = EEPROM.read(OFFSET_MAX_VOLUME);
}

InitValues::~InitValues()
{
}


String *InitValues::getParameterVersion()
{
	return pParameterVersion;
}


void InitValues::writeLabel(int labelOffset, String label)
{
	int laenge = label.length();
	int offset = 0;

	if (laenge > 15)
		laenge = 15;
	for (offset = 0; offset < laenge; offset++)
	{
		EEPROM.write(labelOffset + offset, label[offset]);
	}
	EEPROM.write(labelOffset + offset, 0);
}

String *InitValues::readLabel(int offsetLabel)
{
	char  achBuffer[16];

	for (int i = 0; i < 16; i++)
	{
		achBuffer[i] = EEPROM.read(offsetLabel + i);
		if (achBuffer[i] == 0)
			break;
	}
	achBuffer[15] = 0;
	return new String(achBuffer);
}


void InitValues::writeRc5Commands(int offsetCommands, RC5_COMMAND_VALUES *pRc5CommandsPar)
{
	RC5_COMMAND_VALUES    *pRc5Commands = pRc5CommandsPar;
	int                   offset = 0;

	for (int i = 0; i < anzahlRC5Commands; i++)
	{
		EEPROM.write(offsetCommands + offset, pRc5Commands->command);
		EEPROM.write(offsetCommands + offset + 1, pRc5Commands->value);
		pRc5Commands++;
		offset += 2;
	}
	EEPROM.write(offsetCommands + offset, pRc5Commands->command);
	EEPROM.write(offsetCommands + offset + 1, pRc5Commands->value);
}

RC5_COMMAND_VALUES *InitValues::readRc5Commands(int offsetCommands)
{
	RC5_COMMAND_VALUES    *pRc5Commands = new RC5_COMMAND_VALUES[50];
	RC5_COMMAND_VALUES    *pRc5CommandsStart = pRc5Commands;
	int                   offset = 0;

	//  Serial.println("readRc5Commands");
	//  Serial.println((uint16_t)pRc5Commands, HEX);
	for (int i = 0; i < anzahlRC5Commands; i++)
	{
		pRc5Commands->command = (RC5_COMMANDS)EEPROM.read(offsetCommands + offset);
		pRc5Commands->value = EEPROM.read(offsetCommands + offset + 1);
		pRc5Commands++;
		offset += 2;
	}
	pRc5Commands->command = RC5_END; //(byte)EEPROM.read(offsetCommands + offset);    
	pRc5Commands->value = EEPROM.read(offsetCommands + offset + 1);
	return(pRc5CommandsStart);
}


String *InitValues::getChannelName(int channel)
{
	switch (channel)
	{
	case 0:
		return pLabelChannel1;
		break;

	case 1:
		return pLabelChannel2;
		break;

	case 2:
		return pLabelChannel3;
		break;

	case 3:
		return pLabelChannel4;
		break;

	case 4:
		return pLabelChannel5;
		break;
	}
	return new String("Falscher Kanal");
}


void InitValues::saveNVRamChannel(int channel)
{
	EEPROM.write(OFFSET_CHANNEL, channel);
}


void InitValues::saveNVRamVolumeChannel(int channel, int volumeChannel)
{
	switch (channel)
	{
	case 0:
		volumeChannel1 = volumeChannel;
		EEPROM.write(OFFSET_VOLUME_CHANNEL_0, volumeChannel);
		break;

	case 1:
		volumeChannel2 = volumeChannel;
		EEPROM.write(OFFSET_VOLUME_CHANNEL_1, volumeChannel);
		break;

	case 2:
		volumeChannel3 = volumeChannel;
		EEPROM.write(OFFSET_VOLUME_CHANNEL_2, volumeChannel);
		break;

	case 3:
		volumeChannel4 = volumeChannel;
		EEPROM.write(OFFSET_VOLUME_CHANNEL_3, volumeChannel);
		break;

	case 4:
		volumeChannel5 = volumeChannel;
		EEPROM.write(OFFSET_VOLUME_CHANNEL_4, volumeChannel);
		break;

	}
}



int InitValues::getChannelDefaultValue(int channel)
{
	switch (channel)
	{
	case 0:
		return volumeChannel1;
		break;

	case 1:
		return volumeChannel2;
		break;

	case 2:
		return volumeChannel3;
		break;

	case 3:
		return volumeChannel4;
		break;

	case 4:
		return volumeChannel5;
		break;
	}
	return 0;
}

int InitValues::getMaxVolume()
{
	return maxVolume;
}
