#include <Arduino.h>
#include "RC5.h"

#define                   RC5_INTERRUPT_PIN             20
#define                   RC_REPEATE_MILLIS             110
#define                   RC_SINGLE_MILLIS				400

extern int                anzahlRC5Commands;

volatile bool             firedRC = true;
volatile unsigned long    lastInterruptTime;
volatile unsigned int     command;
volatile static int       toggleBit;
void                      rcIsr();
void                      reset();



RC5::RC5()
{
}


RC5::~RC5()
{
}


void RC5::init(RC5_COMMAND_VALUES *pRc5CommandsPar)
{
	RC::init();
	pRc5Commands = pRc5CommandsPar;
	pinMode(RC5_INTERRUPT_PIN, INPUT);
	attachInterrupt(digitalPinToInterrupt(RC5_INTERRUPT_PIN), rcIsr, CHANGE);
	command = 0;
	toggleBit = 2;
	firedRC = false;
}


Action  *RC5::getAction()
{
	Action    *pAction = new Action();

	pAction->setActionType(NONE);
	if (firedRC)
	{
		RC5_COMMANDS  rc5Command = getRc5Command(command & 0x03f);
		ACTIONTYPE    actionType = getActionFromRc5(rc5Command);

		// Kommando wird verwendet, wenn
		// 1. Das Togglebit sich gendert hat -> neues Kommando
		// 2. Das Togglebit sich nicht gendert hat und das letzte Kommando innerhalb von RC_REPEATE_MILLIS gesendet wurde

		// Neu (Version 1.16):
		// Das Kommando wird nicht verwendet, wenn es ein wiederholtes MUTE, CHANNEL_PLUS, CHANNEL_MINUS, CHANNEL_ABSOLUTE, ACTION_DISPLAY, ACTION_POWER oder VOLUME_CLEAR ist.

		if (!((((command & 0x800) >> 11) == toggleBit) && ((actionType == MUTE) || (actionType == CHANNEL_PLUS) || (actionType == CHANNEL_MINUS) ||
			(actionType == CHANNEL_ABSOLUTE) || (actionType == ACTION_DISPLAY) || (actionType == ACTION_POWER) || (actionType == VOLUME_CLEAR) || 
			(actionType == VOLUME_RESTORE))))
		{

			if (((command & 0x0800) >> 11) != toggleBit)
			{
				pAction->setActionType(actionType);
				pAction->setRcCode(command & 0x03f);
				pAction->setToggleBit((command & 0x0800) >> 11);
				if ((getActionFromRc5(rc5Command) == CHANNEL_ABSOLUTE))
				{
					pAction->setChannelAbsolute(getAbsoluteChannelFromRc5(rc5Command));
				}
				lastCommandReceived = millis();
				lastCommand = command;
				lastCommandSingle = true;
			}
			else if	((((command & 0x0800) >> 11) == toggleBit)
						&& (lastCommand == command) &&
						((lastCommandReceived + (lastCommandSingle == true ? RC_SINGLE_MILLIS : RC_REPEATE_MILLIS)) <= millis()))
			{
				pAction->setActionType(actionType);
				pAction->setRcCode(command & 0x03f);
				pAction->setToggleBit((command & 0x0800) >> 11);
				if ((getActionFromRc5(rc5Command) == CHANNEL_ABSOLUTE))
				{
					pAction->setChannelAbsolute(getAbsoluteChannelFromRc5(rc5Command));
				}
				lastCommandReceived = millis();
				lastCommand = command;
				lastCommandSingle = false;
			}
		}
		toggleBit = ((command & 0x0800) >> 11);
		firedRC = false;
	}
	return pAction;
}


int           RC5::getAbsoluteChannelFromRc5(RC5_COMMANDS rc5Command)
{

	switch (rc5Command)
	{
	case RC5_ONE:
		return 0;
		break;

	case RC5_TWO:
		return 1;
		break;

	case RC5_THREE:
		return 2;
		break;

	case RC5_FOUR:
		return 3;
		break;

	case RC5_FIVE:
		return 4;
		break;
	}
	return 0;
}


RC5_COMMANDS  RC5::getRc5Command(int rc5Value)
{
	RC5_COMMAND_VALUES          *pActualRc5 = pRc5Commands;

	Serial.print("Rc5Commando: ");
	Serial.println(rc5Value);
	for (int i = 0; i < anzahlRC5Commands; i++)
	{
//		Serial.println(pActualRc5->value);
		if (pActualRc5->value == rc5Value)
		{
//			Serial.println("RC5::getRc5Command Kommando gefunden");
//			Serial.println(pActualRc5->value);
			return pActualRc5->command;
		}
		pActualRc5++;
	}
	return RC5_END;
}


ACTIONTYPE RC5::getActionFromRc5(RC5_COMMANDS command)
{
	ACTIONTYPE result = RC_TEST;
	switch (command)
	{
	case RC5_VOLUME_PLUS:
		result = VOLUME_PLUS;
		break;

	case RC5_VOLUME_MINUS:
		result = VOLUME_MINUS;
		break;

	case RC5_ONE:
	case RC5_TWO:
	case RC5_THREE:
	case RC5_FOUR:
	case RC5_FIVE:
		result = CHANNEL_ABSOLUTE;
		break;

	case RC5_CHANNEL_PLUS:
		result = CHANNEL_PLUS;
		break;

	case RC5_CHANNEL_MINUS:
		result = CHANNEL_MINUS;
		break;

	case RC5_MUTE:
		result = MUTE;
		break;

	case RC5_SET:
		result = VOLUME_SAVE;
		break;

	case RC5_POWER:
		result = ACTION_POWER;
		break;

	case RC5_DISPLAY:
		result = ACTION_DISPLAY;
		break;

	case RC5_ZERO:
		result = VOLUME_CLEAR;
		break;

	case RC5_VOLUME_RESTORE:
		result = VOLUME_RESTORE;
		break;

	}
	return result;
}

void          RC5::displayRcCodeReceived()
{
	RC::displayRcCodeReceived();
}

void rcIsr()
{
	if (firedRC == false)
	{
		unsigned long       interruptTime = micros();
		unsigned long       timeSpan = interruptTime - lastInterruptTime;

		if (timeSpan < RC5_MIN_TIME)
		{
			command = 0;
			lastInterruptTime = interruptTime;
		}
		if (timeSpan > RC5_MAX_TIME)
		{
			command = 0;
		}

		unsigned short rc5Bit = digitalRead(RC5_INTERRUPT_PIN) ? 0 : 1;

		if (((command == 0) && rc5Bit) ||
			((timeSpan > RC5_MAX_LONG_PULSE_TIME) && (timeSpan < RC5_MAX_TIME)))
		{
			if (command < 0x2000)
			{
				command <<= 1;
				command |= rc5Bit;
			}
			if (command & 0x2000)
			{
				firedRC = true;
			}
			lastInterruptTime = interruptTime;
		}
	}
}

